@@ -5,6 +5,7 @@
 #' @param attr_str A character string of the same length as x (or of length 1).
 #' @param ignore_na If TRUE, do not add tag if value is NA
 #' @param span_adjacent If TRUE, include adjacent tokens with identical attr_str within the same tag
+#' @param doc_id  If span_adjacent is TRUE, The document ids are required to ensure that tags do not span from one document to another.
 #'
 #' @export
 #'
@@ -21,15 +22,18 @@
 #' add_tag(x, 'span',
 #'         tag_attr(class = "president",
 #'                  style = attr_style(`background-color` = 'rgba(255, 255, 0, 1)')))
-add_tag <- function(x, tag, attr_str=NULL, ignore_na=F, span_adjacent=F) {
+add_tag <- function(x, tag, attr_str=NULL, ignore_na=F, span_adjacent=F, doc_id=NULL) {
   if (!is.null(attr_str)) {
     attr_str = ifelse(is.na(attr_str), yes = if (ignore_na) NA else '',
                                        no = stringi::stri_paste(' ', attr_str, sep=''))
   } else attr_str = if (ignore_na) NA else ''
 
+
   if (span_adjacent) {
-    same_as_next = attr_str == c(attr_str[-1], '#####')
-    same_as_prev = attr_str == c('#####', attr_str[-length(attr_str)])
+
+    if (is.null(doc_id)) stop('If span_adjacent is used, document ids must be given')
+    same_as_next = str_is_lead(attr_str) & str_is_lead(doc_id)
+    same_as_prev = str_is_lag(attr_str) & str_is_lag(doc_id)
 
     x = ifelse(!same_as_prev, stringi::stri_paste('<',tag, attr_str,'>',  x, sep=''), x)
     x = ifelse(!same_as_next, stringi::stri_paste(x, '</',tag,'>', sep=''), x)
@@ -37,6 +41,16 @@ add_tag <- function(x, tag, attr_str=NULL, ignore_na=F, span_adjacent=F) {
   x
 }
 
+str_is_lag <- function(x) {
+  x = as.character(x)
+  c(F, x[-length(x)] == x[-1])
+}
+
+str_is_lead <- function(x) {
+  x = as.character(x)
+  c(x[-1] == x[-length(x)], F)
+}
+
 
 #' HTML tables for meta data per document
 #'
